library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity CamInit is
    Port ( clk : in  STD_LOGIC;
           cam_init : in  STD_LOGIC;
           rs232_used : out  STD_LOGIC;
           rsrcv_int : out  STD_LOGIC);
end CamInit;

architecture Behavioral of CamInit is

	constant sio_cmp : std_logic_vector(15 downto 0) := x"0A2C"; -- 19200 Baud
	signal Baud_clk : std_logic;


begin
	-- Taktteiler fr den Baud-Takt (Senden)
	process(clk) 
		variable sio_cnt : std_logic_vector(15 downto 0):=x"0000";
	begin
		if clk'event and clk='1' then
			if sio_cnt = ('0' & sio_cmp(15 downto 1)) then
				Baud_clk <= '1';
			elsif sio_cnt = sio_cmp then
				Baud_clk <= '0';
				sio_cnt := x"0000";
			end if;
			sio_cnt := sio_cnt + 1;
		end if;
	end process;

	-- Ausgabe der Kamera Startsequenz 

	process(Baud_clk)
		constant cam_01 : std_logic_vector(7 downto 0):=x"01"; -- write
		constant cam_02 : std_logic_vector(7 downto 0):=x"02"; -- read
		constant cam_03 : std_logic_vector(7 downto 0):=x"03"; -- off
		constant cam_04 : std_logic_vector(7 downto 0):=x"04"; -- on
		constant cam_12 : std_logic_vector(7 downto 0):=x"12"; -- register 12
		constant cam_13 : std_logic_vector(7 downto 0):=x"13"; -- value 13
		constant cam_0D : std_logic_vector(7 downto 0):=x"0d"; -- register 0d
		constant cam_C1 : std_logic_vector(7 downto 0):=x"c1"; -- value c1
		constant cam_42 : std_logic_vector(7 downto 0):=x"42"; -- 42
		variable init_cnt : std_logic_vector(15 downto 0):=x"0000";
		variable init_run : std_logic := '0';
	begin
		if Baud_clk'event and Baud_clk='1' then 
			if cam_init ='1' and init_run = '0' then  -- wenn Taste gedrueckt und init laeuft nicht
				init_run :='1';
				init_cnt := x"0000";
			elsif init_run = '1' then -- init laeuft
				case init_cnt is 
					-- Schnittstelle wird benutzt und Startbit
					when x"0000"|x"0100"|x"1000"|x"1010"|x"1020"|x"1030"|x"1100"|x"1120"|x"1130"|x"1140"|x"1150"|x"1160"|x"1170"|x"1200"|x"1220"|x"1230" => 
						rs232_used <= '1'; rsrcv_int <= '0';
					-- off : Kamera ausschalten, da sie an sein koennte
					when x"0001"|x"0002"|x"0003"|x"0004"|x"0005"|x"0006"|x"0007"|x"0008" =>
						rsrcv_int <= cam_03(conv_integer(init_cnt(3 downto 0)-'1'));
					-- 1. Stoppbit
					when x"0009"|x"0109"|x"1009"|x"1019"|x"1029"|x"1039"|x"1109"|x"1129"|x"1139"|x"1149"|x"1159"|x"1169"|x"1179"|x"1209"|x"1229"|x"1239" => 
						rsrcv_int <= '1'; 
					-- 2. Stoppbit
					when x"000A"|x"010A"|x"100A"|x"101A"|x"102A"|x"103A"|x"110A"|x"112A"|x"113A"|x"114A"|x"115A"|x"116A"|x"117A"|x"120A"|x"122A"|x"123A" => 
						rsrcv_int <= '1'; 
					-- Schnittstelle wieder frei
					when x"000B"|x"010B"|x"100B"|x"101B"|x"102B"|x"103B"|x"110B"|x"112B"|x"113B"|x"114B"|x"115B"|x"116B"|x"117B"|x"120B"|x"122B"|x"123B" => 
						rs232_used <= '0';
					-- on : Kamera einschalten
					when x"0101"|x"0102"|x"0103"|x"0104"|x"0105"|x"0106"|x"0107"|x"0108" => 
						rsrcv_int <= cam_04(conv_integer(init_cnt(3 downto 0)-'1'));
					-- write : Schreibkommando an Kamera
					when x"1001"|x"1002"|x"1003"|x"1004"|x"1005"|x"1006"|x"1007"|x"1008"|x"1141"|x"1142"|x"1143"|x"1144"|x"1145"|x"1146"|x"1147"|x"1148" => 
						rsrcv_int <= cam_01(conv_integer(init_cnt(3 downto 0)-'1'));
					-- send 42 : Wert 42 an Kamera schicken
					when x"1011"|x"1012"|x"1013"|x"1014"|x"1015"|x"1016"|x"1017"|x"1018"|x"1121"|x"1122"|x"1123"|x"1124"|x"1125"|x"1126"|x"1127"|x"1128"|x"1151"|x"1152"|x"1153"|x"1154"|x"1155"|x"1156"|x"1157"|x"1158"|x"1221"|x"1222"|x"1223"|x"1224"|x"1225"|x"1226"|x"1227"|x"1228" => 
						rsrcv_int <= cam_42(conv_integer(init_cnt(3 downto 0)-'1'));
					-- send 12 : Register 12 auswaehlen
					when x"1021"|x"1022"|x"1023"|x"1024"|x"1025"|x"1026"|x"1027"|x"1028"|x"1131"|x"1132"|x"1133"|x"1134"|x"1135"|x"1136"|x"1137"|x"1138" => 
						rsrcv_int <= cam_12(conv_integer(init_cnt(3 downto 0)-'1'));
					-- send 13 : Wert 13 schicken
					when x"1031"|x"1032"|x"1033"|x"1034"|x"1035"|x"1036"|x"1037"|x"1038" => 
						rsrcv_int <= cam_13(conv_integer(init_cnt(3 downto 0)-'1'));
					-- send 12 : Register 0d auswaehlen
					when x"1161"|x"1162"|x"1163"|x"1164"|x"1165"|x"1166"|x"1167"|x"1168"|x"1231"|x"1232"|x"1233"|x"1234"|x"1235"|x"1236"|x"1237"|x"1238" => 
						rsrcv_int <= cam_0D(conv_integer(init_cnt(3 downto 0)-'1'));
					-- send 13 : Wert c1 schicken
					when x"1171"|x"1172"|x"1173"|x"1174"|x"1175"|x"1176"|x"1177"|x"1178" => 
						rsrcv_int <= cam_C1(conv_integer(init_cnt(3 downto 0)-'1'));
					-- read : Lesekommando an Kamera
					   when x"1101"|x"1102"|x"1103"|x"1104"|x"1105"|x"1106"|x"1107"|x"1108"|x"1201"|x"1202"|x"1203"|x"1204"|x"1205"|x"1206"|x"1207"|x"1208" => 
					--	rsrcv_int <= cam_02(conv_integer(init_cnt(3 downto 0)-'1'));
					-- Zaehler am Ende, bereit fuer neues Init
					when x"FFFF" =>
						if cam_init ='0' then
							init_run := '0';
						end if;
					when others => null;
				end case;
				-- Zaehler anhalten, um Ueberlauf zu vermeiden
				if init_cnt < x"FFFF" then 
					init_cnt := init_cnt +'1';
				end if;
			end if;
		end if;
	end process;

end Behavioral;



